from indice_pollution.extensions import db

class Zone(db.Model):
    __table_args__ = {"schema": "indice_schema"}

    id = db.Column(db.Integer, primary_key=True)
    type = db.Column(db.String)
    code = db.Column(db.String)


    @classmethod
    def get(cls, code, type_):
        return Zone.query.filter_by(code=code, type=type_).first()



from sqlalchemy.orm import relationship
from indice_pollution.history.models.commune import Commune
from indice_pollution.extensions import db
from datetime import datetime
from indice_pollution.helpers import today
from sqlalchemy import  Date, text

class EpisodePollution(db.Model):
    __table_args__ = {"schema": "indice_schema"}

    zone_id: int = db.Column(db.Integer, db.ForeignKey('indice_schema.zone.id'), primary_key=True, nullable=True)
    zone = relationship("indice_pollution.history.models.zone.Zone")
    date_ech: datetime = db.Column(db.DateTime, primary_key=True, nullable=True)
    date_dif: datetime = db.Column(db.DateTime, primary_key=True, nullable=True)
    code_pol: int = db.Column(db.Integer, primary_key=True)
    etat: str = db.Column(db.String)
    com_court: str = db.Column(db.String)
    com_long: str = db.Column(db.String)

    @classmethod
    def get(cls, insee=None, code_epci=None, date_=None):
        if insee:
            zone_subquery = Commune.get_query(insee=insee).limit(1).with_entities(Commune.zone_pollution_id).subquery()
        date_ = date_ or today()
        query = \
            cls.query.filter(
                cls.date_ech.cast(Date)==date_,
                cls.zone_id.in_(zone_subquery)
            )\
            .order_by(cls.date_dif.desc())
        return query.all()

    @classmethod
    def bulk_query(cls, insees=None, codes_epci=None, date_=None):
        return text(
            """
            SELECT
                DISTINCT ON (e.zone_id, e.date_ech) e.*, e.date_ech e, c.insee
            FROM
                indice_schema.episode_pollution e
                LEFT JOIN indice_schema.commune c ON c.zone_pollution_id = e.zone_id
                WHERE c.insee = ANY(:insees) AND date_ech=:date_ech
                ORDER BY e.zone_id, e.date_ech, e.date_dif DESC
            """
        ).bindparams(
            date_ech=date_,
            insees=insees
        )
    
    @classmethod
    def bulk(cls, insees=None, codes_epcis=None, date_=None):
        return db.session.execute(cls.bulk_query(insees=insees, date_=date_))


    @property
    def lib_pol(self):
        return {
            1: 'Dioxyde de soufre',
            3: 'Dioxyde d’azote',
            4: 'Monoxyde de carbone',
            5: 'Particules PM 10',
            7: 'Ozone',
            8: 'Dioxyde d’azote',
            24: 'Particules PM10',
            39: 'Particules PM25',
        }.get(self.code_pol)

    def dict(self):
        return {
            "code_pol": f"{self.code_pol:02}",
            "code_zone": self.zone.code,
            "com_court": self.com_court,
            "com_long": self.com_long,
            "date": self.date_ech.date().isoformat(),
            "date_ech": self.date_ech.date().isoformat(),
            "date_dif": self.date_dif.date().isoformat(),
            "etat": self.etat,
            "lib_pol": self.lib_pol
        }


from flask import current_app, render_template

@current_app.route('/')
def root():
    return render_template('index.html')

@current_app.route('/ville/<insee>')
def ville(insee):
    return render_template('indice.html', insee=insee)


from sqlalchemy.sql.elements import _corresponding_column_or_error
from indice_pollution.extensions import db
import json
from .commune import Commune
from flask import current_app

class EpisodeHistory(db.Model):
    __table_args__ = {"schema": "indice_schema"}

    id = db.Column(db.Integer, primary_key=True)
    date_ = db.Column(db.Date, nullable=False)
    polluant = db.Column(db.String)
    code_zone = db.Column(db.String, nullable=False)
    features = db.Column(db.JSON, nullable=False)

    def __init__(self, features) -> None:
        try:
            super().__init__(
                date_=features['date'],
                polluant=features['code_pol'],
                code_zone=str(features['code_zone']),
                features=features
            )
        except KeyError as e:
            raise e

    @classmethod
    def get(cls, date_, code_zone=None, polluant=None):
        query = db.session.query(cls).filter_by(date_=date_)
        if code_zone:
            query = query.filter_by(code_zone=code_zone)
        if polluant:
            query = query.filter_by(polluant=polluant)
        return query.first()
    
    @classmethod
    def get_bulk(cls, date_, insee_list):
        insee_list = set(insee_list)
        insee_code_zone_list = [
            (c.insee, c.code_zone)
            for c in Commune.query.filter(Commune.insee.in_(insee_list), Commune.insee != None)
        ]
        diff = insee_list - set([c[0] for c in insee_code_zone_list])
        if len(diff) > 0:
            for insee in diff:
                if len(insee) == 5:
                    commune = Commune.get(insee)
                    if commune and commune.code_zone:
                        insee_code_zone_list.append(
                            (commune.insee, commune.code_zone)
                        )
                elif len(insee) == 2:
                    insee_code_zone_list.append((insee, insee))
        code_zone_list = [c[1] for c in insee_code_zone_list]

        return cls.query\
            .filter(cls.date_==date_, cls.code_zone.in_(set(code_zone_list)))\
            .all()

    @classmethod
    def get_after(cls, date_, insee):
        if len(insee) == 5:
            code_zone = Commune.get(insee).code_zone
        else:
            code_zone = insee
        return [v.features for v in cls.query.filter(cls.date_ >= date_, cls.code_zone==code_zone).all()]

    @classmethod
    def get_or_create(cls, date_, insee=None, code_zone=None, features=None):
        if not code_zone:
            if insee and len(insee) == 5:
                commune = Commune.get(insee)
                if not commune.code_zone and 'code_zone' in features:
                    commune.code_zone = str(features['code_zone'])
                    db.session.add(commune)
                    db.session.commit()
                code_zone = commune.code_zone
            if insee and len(insee) == 2:
                code_zone = insee
        if type(features) == dict:
            polluant = features.get("code_pol") or features.get("code_polluant")
        else:
            polluant = None
        if code_zone and polluant:
            result = cls.get(date_, code_zone=code_zone, polluant=str(polluant))
            if result:
                return result
        result = cls.get(date_, code_zone=code_zone, polluant=str(polluant))
        if result:
            return result
        try:
            result = cls(features)
        except KeyError as e:
            current_app.logger.error(f"Unable to create episode with {features}")
            current_app.logger.error(e)
            return None

        if result.date_ == date_:
            db.session.add(result)
            db.session.commit()
            return result
        return None


from . import create_app
import sentry_sdk
from sentry_sdk.integrations.celery import CeleryIntegration
from sentry_sdk.integrations.redis import RedisIntegration
import os

if os.getenv('SENTRY_DSN'):
    sentry_sdk.init(
        dsn=os.getenv('SENTRY_DSN'),
        integrations=[CeleryIntegration(), RedisIntegration()],
        traces_sample_rate=1.0
    )
app = create_app()

from .extensions import celery #noqa


from indice_pollution.extensions import db
import requests

class Region(db.Model):
    __table_args__ = {"schema": "indice_schema"}
    __tablename__ = 'region'

    id = db.Column(db.Integer, primary_key=True)
    nom = db.Column(db.String)
    code = db.Column(db.String)

    @classmethod
    def get(cls, code):
        return db.session.query(cls).filter_by(code=code).first() or cls.get_and_init_from_api(code)

    @classmethod
    def get_and_init_from_api(cls, code):
        res_api = cls.get_from_api(code)
        r = cls(**res_api)        
        db.session.add(r)
        db.session.commit()
        return r

    @classmethod
    def get_from_api(cls, code):
        r = requests.get(f'https://geo.api.gouv.fr/regions/{code}?fields=nom,code')
        r.raise_for_status()
        return r.json()


from setuptools import find_packages, setup

DEPENDENCIES = [
    'alembic',
    'celery',
    'celery[redis]',
    'flask',
    'flask-celery',
    'flask-cors',
    'Flask-Manage-Webpack',
    'flask-migrate',
    'flask-sqlalchemy',
    'requests',
    'python-dateutil',
    'orjson',
    'pytz',
    'beautifulsoup4',
    'html5lib',
    'psycopg2',
    'unidecode',
    'sentry-sdk[flask]',
    'redis'
]

setup(
    name='indice_pollution',
    version='0.22.1',
    description='API giving air pollution level in France',
    url='https://github.com/l-vincent-l/indice_pollution',
    download_url='https://github.com/l-vincent-l/indice_pollution/archive/0.1.2.tar.gz',
    author='Vincent Lara',
    author_email='vincent.lara@beta.gouv.fr',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3.8'
    ],
    keywords='air quality aasqa atmo iqa',
    packages=find_packages(),
    install_requires=DEPENDENCIES
)




from indice_pollution.extensions import db
from sqlalchemy.orm import relationship
from indice_pollution.history.models import Commune

class EPCI(db.Model):
    __table_args__ = {"schema": "indice_schema"}
    __tablename__ = "epci"

    id = db.Column(db.Integer, primary_key=True)
    code = db.Column(db.String())
    label = db.Column(db.String())
    departement_id = db.Column(db.Integer, db.ForeignKey("indice_schema.departement.id"))
    departement = relationship("indice_pollution.history.models.departement.Departement")
    zone_id = db.Column(db.Integer, db.ForeignKey("indice_schema.zone.id"))
    zone = relationship("indice_pollution.history.models.zone.Zone")

    @classmethod
    def get(cls, code=None, insee=None):
        return cls.get_query(code, insee).first()

    @classmethod
    def get_query(cls, code=None, insee=None):
        if code:
            return cls.query.filter_by(code=code)
        elif insee:
            subquery = Commune.get_query(insee).with_entities(Commune.epci_id)
            return cls.query.filter(cls.id.in_(subquery))

    @classmethod
    def bulk_query(cls, codes=None, insees=None):
        if codes:
            return cls.query.filter(cls.code.in_(codes))
        elif insees:
            subquery = Commune.bulk_query(insees=insees).with_entities(Commune.epci_id)
            return cls.query.filter(cls.id.in_(subquery))